tdelta as an alias for (make-time-delta)
[8sync.git] / loopy.scm
index 70d2d08ba7e2b42083452a3d5f30e8fe8f93f508..3954fea34a853c82e49726eaf40329f2a4509ba7 100644 (file)
--- a/loopy.scm
+++ b/loopy.scm
@@ -18,7 +18,7 @@
             time-< time-= time-<= time-+
 
             <time-delta>
-            make-time-delta time-delta?
+            make-time-delta tdelta time-delta?
             time-delta-sec time-delta-usec
 
             <schedule>
@@ -90,7 +90,7 @@
 
 ;;; This is where we handle timed events for the future
 
-;; This section totally borrows from SICP
+;; This section totally borrows from the ideas in SICP
 ;; <3 <3 <3
 
 ;; NOTE: time is a cons of (seconds . microseconds)
 (define* (make-time-delta sec #:optional usec)
   (make-time-delta-intern sec (or usec 0)))
 
+(define tdelta make-time-delta)
+
 (define (time-+ time time-delta)
   (cons (+ (car time) (time-delta-sec time-delta))
         (+ (cdr time) (time-delta-usec time-delta))))
 ;;   mutates AND is worst case of O(n) in both space and time :(
 ;;   but at least it'll be reasonably easy to refactor to
 ;;   a more functional setup?
-(define (schedule-add! time proc schedule)
+(define (schedule-add! schedule time proc)
   (let ((time (time-segment-right-format time)))
     (define (new-time-segment)
       (let ((new-segment
 
 (define* (start-agenda agenda #:optional stop-condition)
   (let loop ((agenda agenda))
-    (let ((new-agenda   
+    (let ((agenda   
            ;; @@: Hm, maybe here would be a great place to handle
            ;;   select'ing on ports.
            ;;   We could compose over agenda-run-once and agenda-read-ports
              (agenda-run-once agenda))))
       (if (and stop-condition (stop-condition agenda))
           'done
-          (let ((updated-agenda
-                 ;; Adjust the agenda's time just in time
-                 ;; We do this here rather than in agenda-run-once to make
-                 ;; agenda-run-once's behavior fairly predictable
-                 (set-field new-agenda (agenda-time) (gettimeofday))))
-            (loop updated-agenda))))))
+          (let* ((new-time (gettimeofday))
+                 (agenda
+                  ;; Adjust the agenda's time just in time
+                  ;; We do this here rather than in agenda-run-once to make
+                  ;; agenda-run-once's behavior fairly predictable
+                  (set-field agenda (agenda-time) new-time)))
+            ;; Update the agenda's current queue based on
+            ;; currently applicable time segments
+            (add-segments-contents-to-queue!
+             (schedule-extract-until! (agenda-schedule agenda) new-time)
+             (agenda-queue agenda))
+            (loop agenda))))))
 
 (define (agenda-run-once agenda)
   "Run once through the agenda, and produce a new agenda
@@ -336,11 +344,21 @@ based on the results"
              (proc-result (call-proc proc))
              (enqueue
               (lambda (run-request)
-                (cond
-                 ((run-request-when run-request)
-                  (error "TODO"))
-                 (else
-                  (enq! next-queue (run-request-proc run-request)))))))
+                (define (schedule-at! time proc)
+                  (schedule-add! (agenda-schedule agenda) time proc))
+                (let ((request-time (run-request-when run-request)))
+                  (match request-time
+                    ((? time-delta? time-delta)
+                     (let ((time (time-+ (agenda-time agenda)
+                                         time-delta)))
+                       (schedule-at! time (run-request-proc run-request))))
+                    ((? integer? sec)
+                     (let ((time (cons sec 0)))
+                       (schedule-at! time (run-request-proc run-request))))
+                    (((? integer? sec) . (? integer? usec))
+                     (schedule-at! request-time (run-request-proc run-request)))
+                    (#f
+                     (enq! next-queue (run-request-proc run-request))))))))
         ;; @@: We might support delay-wrapped procedures here
         (match proc-result
           ;; TODO: replace procedure with something that indicates
@@ -354,6 +372,7 @@ based on the results"
             new-procs))
           ;; do nothing
           (_ #f))))
-    ;; TODO: Selecting on ports would happen here?
+    ;; TODO: Alternately, we could return the next-queue
+    ;;   along with changes to be added to the schedule here?
     ;; Return new agenda, with next queue set
     (set-field agenda (agenda-queue) next-queue)))